// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <string.h>
#include "../../plugins/fit/fit.h"
#include "../../sim.h"
double lib_error(struct fitvars *config,char* name,char * path)
{
double error=0.0;
char tempstring[1000];
double mul;
struct istruct exp;
struct istruct sim;

char exp_file[500];
char sim_file[500];
char config_file[500];
char out_name_exp[1000];
char out_name_sim[1000];
char out_name_delta[1000];

sprintf(exp_file,"%s./exp/%s/imps_j.dat",path,name);
sprintf(sim_file,"%s./sim/%s/imps_j.dat",path,name);
sprintf(out_name_exp,"./error_%s_exp.dat",name);
sprintf(out_name_sim,"./error_%s_sim.dat",name);
sprintf(out_name_delta,"./error_%s_delta.dat",name);
sprintf(config_file,"%s./exp/%s/fit.inp",path,name);

inter_load(&exp,exp_file);
inter_load(&sim,sim_file);
inter_sort(&exp);
inter_sort(&sim);
inter_save(&exp,"./a.dat");
inter_save(&sim,"./b.dat");

double start=0.3;
double stop=0.7;

	FILE *info=fopen(config_file,"r");
	if (info==NULL)
	{
		printf("Problem with %s\n",config_file);
		getchar();
	}
	fscanf(info,"%s",tempstring);
	fscanf(info,"%le",&start);
	fscanf(info,"%s",tempstring);
	fscanf(info,"%le",&stop);
	fscanf(info,"%s",tempstring);
	fscanf(info,"%le",&mul);
	fscanf(info,"%s",tempstring);

	if (strcmp("#end",tempstring)!=0)
	{
		printf("Problem with fit.inp in %s!\n","./exp/imps0/");
		getchar();
	}
	fclose(info);

struct istruct mul_data;
inter_copy(&mul_data,&exp,TRUE);
inter_set_value(&mul_data,mul);

if (exp.x[0]>start) start=exp.x[0];
if (sim.x[0]>start) start=sim.x[0];

if (exp.x[exp.len-1]<stop) stop=exp.x[exp.len-1];
if (sim.x[sim.len-1]<stop) stop=sim.x[sim.len-1];


inter_chop(&exp,start,stop);
inter_chop(&sim,start,stop);

double min=exp.data[0];
inter_sub_double(&exp,min);
inter_sub_double(&sim,min);

double max=0.0;

max=inter_norm_to_one_range(&exp,start,stop);
inter_rescale(&sim,1.0, 1.0/max);

inter_save(&exp,out_name_exp);
inter_save(&sim,out_name_sim);

error=inter_get_diff(out_name_delta,&exp,&sim,start,stop,&mul_data);

inter_free(&exp);
inter_free(&sim);
inter_free(&mul_data);
return error;
}
