// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <string.h>
#include "../../plugins/fit/fit.h"
#include "../../sim.h"

double lib_error(struct fitvars *config,char* name,char * path)
{
char exp_name[40];
char sim_name[40];
char fit_config_name[40];
//char sim_zero_volt_name[40];
char sim_name_delta[40];
char out_name_exp[40];
char out_name_sim[40];
char out_name_delta[40];

char temp[1000];
double ver;
double mul;
double time_shift;
int zero;
double stop=0.0;
double end_data=0.0;

FILE *file;
sprintf(fit_config_name,"%s./exp/%s/fit.inp",path,name);


file=fopen(fit_config_name,"r");
fscanf(file,"%s",temp);
fscanf(file,"%le",&mul);

fscanf(file,"%s",temp);
fscanf(file,"%le",&time_shift);

fscanf(file,"%s",temp);
fscanf(file,"%d",&zero);

fscanf(file,"%s",temp);
fscanf(file,"%lf",&end_data);

fscanf(file,"%s",temp);
fscanf(file,"%lf",&(ver));
if (ver!=1.0)
{
	printf("File compatability problem %s\n",fit_config_name);
	getchar();
}

fscanf(file,"%s",temp);
if (strcmp("#end",temp)!=0)
{
	printf("Problem with %s file!\n",fit_config_name);
	getchar();
}
fclose(file);

sprintf(exp_name,"%s./exp/%s/stark.dat",path,name);
//sprintf(sim_zero_volt_name,"%s./sim/stark_zero%d/stark_stark_mode.dat",path,zero);
sprintf(sim_name,"%s./sim/%s/stark_rx_mode_norm.dat",path,name);
sprintf(sim_name_delta,"%s./sim/%s/stark_stark_mode_delta.dat",path,name);
sprintf(out_name_exp,"./error_%s_exp.dat",name);
sprintf(out_name_sim,"./error_%s_sim.dat",name);
sprintf(out_name_delta,"./error_%s_delta.dat",name);

char plot_file_name[1000];
sprintf(plot_file_name,"%s./exp/%s/plot_file.dat",path,name);
FILE *plot;
plot=fopen(plot_file_name,"w");

sprintf(temp,"'%s' with lp title '%s',\\\n",exp_name,name);
fprintf(plot,"%s",temp);
sprintf(temp,"'%s' with l lw 2 title '%s',\\\n",sim_name_delta,name);
fprintf(plot,"%s",temp);
fclose(plot);


struct istruct exp;
struct istruct sim;

//struct istruct sim_zero_volt;

double start=0.0;
double error=0.0;
double max=0.0;
inter_load(&exp,exp_name);
inter_load(&sim,sim_name);

start=exp.x[0];
if (start<sim.x[0]) start=sim.x[0];

inter_chop(&exp,start,end_data);
inter_chop(&sim,start,end_data);

struct istruct mul_data;
inter_copy(&mul_data,&exp,TRUE);
inter_set_value(&mul_data,mul);


inter_add_x(&exp,time_shift);
inter_sort(&exp);


int stark_diff=0;	//random patch to make it compile
if (stark_diff>0)
{
	//inter_load(&sim_zero_volt,sim_zero_volt_name);
	//inter_sub(&sim,&sim_zero_volt);
	inter_save(&sim,sim_name_delta);
	//inter_free(&sim_zero_volt);
}

//printf("%ld %s %le %le %le\n",sim.len,sim_name,sim.data[10],sim.data[0],min);



inter_sub_double(&sim,sim.data[0]);
inter_sub_double(&exp,exp.data[0]);

inter_purge_zero(&sim);
inter_purge_zero(&exp);



stop=exp.x[exp.len-1];
if (stop>sim.x[sim.len-1]) stop=sim.x[sim.len-1];


max=inter_get_max(&exp);
inter_div_double(&exp,max);
inter_div_double(&sim,max);


//printf("%s\n",exp_name);
//printf("%s\n",sim_name);

//getchar();

inter_save(&exp,out_name_exp);
inter_save(&sim,out_name_sim);

error=inter_get_diff(out_name_delta,&exp,&sim,start,stop,&mul_data);

inter_free(&exp);
inter_free(&sim);
inter_free(&mul_data);
return error;
}
