// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <zip.h>
#include <unistd.h>
#include <fcntl.h>

#include "inp.h"
#include "util.h"
#include "code_ctrl.h"
#include "true_false.h"

#ifdef enable_lock
#include "./plugins/lock/lock.h"
#endif

void inp_reset_read(struct inp_file *in)
{
in->pos=0;
}

char* inp_get_string(struct inp_file *in)
{
int i;
static char ret[100];
memset(ret, 0, 100);
int ii=0;
if (in->pos>=in->fsize)
{
	return NULL;
}

for (i=in->pos;i<in->fsize;i++)
{
	if ((in->data[i]=='\n')||(in->data[i]==0))
	{
		ret[ii]=0;
		in->pos++;
		break;
	}

	ret[ii]=in->data[i];
	ii++;
	in->pos++;

}

return ret;
}




void inp_read_buffer(char **buf, long *len,char *full_file_name)
{

//sprintf(full_file_name,"%s/%s",path,file);
//#ifndef windows
//if ( access( full_file_name, F_OK ) != -1 )
FILE *f = fopen(full_file_name, "rb");
if (f!=NULL)
{
	fseek(f, 0, SEEK_END);
	*len = ftell(f);
	fseek(f, 0, SEEK_SET);

	*buf = malloc(((*len) + 2)*sizeof(char));
	memset(*buf, 0, ((*len) + 2)*sizeof(char));
	fread(*buf, *len, 1, f);
	fclose(f);

//#ifndef windows
}else
{
	char zip_path[1000];
	char *file_path=get_dir_name_from_path(full_file_name);
	char *file_name=get_file_name_from_path(full_file_name);
//printf("%s %s %s\n",full_file_name,file_path,file_name);
	sprintf(zip_path,"%s/sim.opvdm",file_path);
	int err = 0;
	struct zip *z = zip_open(zip_path, 0, &err);

	if (z!=NULL)
	{
		//Search for the file of given name
		struct zip_stat st;
		zip_stat_init(&st);
		int ret=zip_stat(z, file_name, 0, &st);

		if (ret==0)
		{
			//printf ("Read zip file!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");

			//Alloc memory for its uncompressed contents
			*len=st.size*sizeof(char);
			*buf = (char *)malloc(*len);

			//Read the compressed file
			struct zip_file *f = zip_fopen(z, file_name, 0);
			zip_fread(f, *buf, st.size);
			zip_fclose(f);

		}else
		{
		 	ewe("File %s not found\n", file_name);
		}
		zip_close(z);
	}else
	{
		ewe("error in config file %s",full_file_name);
	}


}
//#endif

}

#ifdef enable_lock
int inp_aes_load(struct inp_file *in,char *path,char *file,char *key)
{
	long int buf_len=0;
	char *buf;

	sprintf(in->full_name,"%s/%s",path,file);

	FILE *f = fopen(in->full_name, "rb");
	if (f==NULL)
	{
		return -1;
	}
	fseek(f, 0, SEEK_END);
	buf_len = ftell(f);
	fseek(f, 0, SEEK_SET);

	buf = malloc((buf_len)*sizeof(char));
	fread(buf, buf_len, 1, f);
	fclose(f);

	aes(&(in->data),&(in->fsize),buf,buf_len,FALSE,key);
	free(buf);
	//printf("%ld %ld %s\n",buf_len,in->fsize,key);
	return 0;
}
#endif

void inp_init(struct inp_file *in)
{
strcpy(in->full_name,"");
in->data=NULL;
in->fsize=0;
in->pos=0;
in->edited=FALSE;
}

void inp_load_from_path(struct inp_file *in,char *path,char *file)
{
char temp[1000];
sprintf(temp,"%s/%s",path,file);
inp_load(in,temp);
}

void inp_load(struct inp_file *in,char *file)
{
in->pos=0;

if (strcmp(in->full_name,file)!=0)
{
	//printf("Reload %s %s\n",in->full_name,file);
	if (in->data!=NULL)
	{
		inp_free(in);
	}

	strcpy(in->full_name,file);
	inp_read_buffer(&(in->data),&(in->fsize),file);

	in->edited=FALSE;
}

}

void inp_replace(struct inp_file *in,char *token, char *text)
{
char *temp = malloc(in->fsize + 100);
memset(temp, 0, in->fsize + 100);
char *line;
int len=0;

line = strtok(in->data, "\n");
int found=FALSE;

while(line)
{
	if (strcmp(line,token)!=0)
	{
		strcat(temp,line);
		strcat(temp,"\n");
	}else
	{
		strcat(temp,line);
		strcat(temp,"\n");
		strcat(temp,text);
		strcat(temp,"\n");
		line  = strtok(NULL, "\n");
		found=TRUE;
	}
	line  = strtok(NULL, "\n");
}


len=strlen(temp);
in->fsize=len;

if (found==TRUE)
{
	in->edited=TRUE;
}

in->data=realloc(in->data,(len+1)*sizeof(char));
memcpy(in->data, temp, (len+1)*sizeof(char));
free(temp);
}

void inp_save(struct inp_file *in)
{

	if (in->edited==TRUE)
	{
		int out_fd = open(in->full_name, O_WRONLY | O_CREAT,0644);
		if (out_fd==-1)
		{
			ewe("File %s can not be opened\n",in->full_name);
		}

	    write(out_fd, in->data, in->fsize*sizeof(char));
		close(out_fd);
		//printf("%s\n",in->data);
		in->edited=FALSE;

	}

}
void inp_free(struct inp_file *in)
{

	inp_save(in);

	free(in->data);
	inp_init(in);
}

void inp_search_double(struct inp_file *in,double* out,char* token)
{
sscanf(inp_search(in,token),"%le",out);
}

void inp_search_int(struct inp_file *in,int* out,char* token)
{
sscanf(inp_search(in,token),"%d",out);
}

void inp_search_string(struct inp_file *in,char* out,char* token)
{
strcpy(out,inp_search(in,token));
}

void inp_check(struct inp_file *in,double ver)
{
double read_ver=0.0;
	inp_reset_read(in);
	char *line=inp_get_string(in);
	while(line!=NULL)
	{

		if (strcmp(line,"#ver")==0)
		{
			line  = inp_get_string(in);
			sscanf(line,"%le",&read_ver);

			if (ver!=read_ver)
			{
				ewe("File compatability problem %s\n",in->full_name);
			}
			line  = inp_get_string(in);

			if ((line==NULL)||(strcmp(line,"#end")!=0))
			{
				ewe("#end token missing %s\n",in->full_name);
			}

			return;
		}

		line  = inp_get_string(in);
	}
ewe("Token #ver not found in %s\n",in->full_name);
return;
}

char* inp_search_part(struct inp_file *in,char *token)
{
	inp_reset_read(in);
	char * line = inp_get_string(in);
	while(line!=NULL)
	{
		//printf("'%s' '%s'\n", line,token);
		if (cmpstr_min(line,token)==0)
		{
			return line;
		}

		line  = inp_get_string(in);
	}

return NULL;
}

char* inp_search(struct inp_file *in,char *token)
{
	inp_reset_read(in);
	char * line = inp_get_string(in);
	while(line!=NULL)
	{
		//printf("'%s' '%s'\n", line,token);
		if (strcmp(line,token)==0)
		{
			line  = inp_get_string(in);
			return line;
		}

		line  = inp_get_string(in);
	}
ewe("Token %s not found in file %s",token,in->full_name);
exit(0);
return NULL;
}


