// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../../sim.h"
#include "stark_spectrum.h"
#include "dump.h"
#include <math.h>
#include "../../elec_plugins.h"
#include "../../inp.h"
#include "../../buffer.h"

static int unused __attribute__((unused));

void sim_stark_spectrum(struct device *in)
{
printf("entered stark_spectrum\n");
struct stark_spectrum config;
stark_spectrum_load_config(&config,in);





struct istruct abs;
inter_init(&abs);
inter_load(&abs,"./phys/redf/juan_alpha.inp");

struct istruct dabs;
inter_init(&dabs);
inter_load(&dabs,"./phys/redf/juan_alpha.inp");

struct istruct ddabs;
inter_init(&ddabs);
inter_load(&ddabs,"./phys/redf/juan_alpha.inp");

inter_smooth(&abs,&dabs,10);
inter_smooth(&abs,&dabs,10);
inter_smooth(&abs,&dabs,10);
inter_smooth(&abs,&dabs,10);

int i;

for (i=0;i<abs.len;i++)
{
	abs.x[i]=6.62606957e-34*3e8/abs.x[i]/1.6e-19;
	dabs.x[i]=6.62606957e-34*3e8/dabs.x[i]/1.6e-19;
	ddabs.x[i]=6.62606957e-34*3e8/ddabs.x[i]/1.6e-19;
}
double shift=0.0;
inter_add_x(&abs,shift);
inter_add_x(&dabs,shift);
inter_add_x(&ddabs,shift);

double max_a=inter_get_max(&abs);

inter_deriv(&dabs,&abs);
double max_b=inter_get_max(&dabs);

inter_deriv(&ddabs,&dabs);
double max_c=inter_get_max(&ddabs);

inter_mul(&abs,config.a);
inter_save_a(&abs,in->outputpath,"out.dat");

inter_mul(&dabs,config.b);
inter_save_a(&dabs,in->outputpath,"dout.dat");


for (i=ddabs.len-20;i<ddabs.len;i++)
{
	ddabs.data[i]=ddabs.data[ddabs.len-20];
}


inter_mul(&ddabs,config.c);
inter_save_a(&ddabs,in->outputpath,"ddout.dat");

inter_deriv(&ddabs,&dabs);

inter_add(&abs,&dabs);
inter_add(&abs,&ddabs);
inter_save_a(&abs,in->outputpath,"stark_spectrum.dat");

inter_free(&abs);
inter_free(&dabs);
inter_free(&ddabs);
printf("max = %f %f %f\n",1.0/max_a,1.0/max_b,1.0/max_c);
}




void stark_spectrum_load_config(struct stark_spectrum* in,struct device *dev)
{
struct inp_file inp;
inp_init(&inp);
inp_load_from_path(&inp,dev->inputpath,"stark_spectrum.inp");
inp_check(&inp,1.0);
inp_search_double(&inp,&(in->a),"#stark_spectrum_a");
inp_search_double(&inp,&(in->b),"#stark_spectrum_b");
inp_search_double(&inp,&(in->c),"#stark_spectrum_c");
inp_free(&inp);
}
