// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../../sim.h"
#include "../jv/jv.h"
#include "dump.h"
#include <math.h>
#include "../../elec_plugins.h"
#include "ntricks.h"
#include "gui_hooks.h"
#include "../../inp.h"
#include "sun_voc.h"
#include "../../buffer.h"
#include "../../dynamic_store.h"

void solve_sun_voc(struct device *in)
{
int clever_contacts=in->config_kl_in_newton;

if (clever_contacts==TRUE)
{
	in->kl_in_newton=TRUE;
	in->newton_aux=&newton_aux_voc;
	solver_realloc(in);
	solve_all(in);
}else
{
	in->kl_in_newton=FALSE;
	solver_realloc(in);
	newton_sun_voc_ittr(in);
}


}

double newton_sun_voc_ittr(struct device *in)
{
double clamp=0.05;
double step=0.01;
double e0;
double e1;
double i0;
double i1;
double deriv;


solve_all(in);
i0=get_I(in);
e0=fabs(in->Vapplied/in->Rshunt+in->Vapplied/(1e6+in->Rcontact)+i0);

in->Vapplied+=step;
solve_all(in);
i1=get_I(in);
e1=fabs(in->Vapplied/in->Rshunt+in->Vapplied/(1e6+in->Rcontact)+i1);

//printf("%le\n",e1);
deriv=(e1-e0)/step;
step=-e1/deriv;
//step=step/(1.0+fabs(step/clamp));
in->Vapplied+=step;
int count=0;
int max=100;
do
{
	e0=e1;
	solve_all(in);
	i1=get_I(in);

	e1=fabs(in->Vapplied/in->Rshunt+in->Vapplied/(1e6+in->Rcontact)+i1);
	printf("error=%le Vapplied=%le \n",e1,in->Vapplied);
	deriv=(e1-e0)/step;
	step=-e1/deriv;
	//double clamp=0.01;
	//if (e1<clamp) clamp=e1/100.0;
	//step=step/(1.0+fabs(step/clamp));
	step=step/(1.0+fabs(step/clamp));
	in->Vapplied+=step;
	if (count>max)
	{
		break;
	}
	count++;
}while(e1>1e-11);

return;
}

void newton_aux_voc(struct device *in,double V,double* i,double* didv,double* didphi,double* didxil,double* didxipl,double* didphir,double* didxir,double* didxipr)
{
#ifdef enable_clever_contacts

double i0=*i;
double didv0=*didv;
double didphi0=*didphi;
double didxil0=*didxil;
double didxipl0=*didxipl;
double didphir0=*didphir;
double didxir0=*didxir;
double didxipr0=*didxipr;

*i=(V/in->Rshunt+V/(1e6+in->Rcontact)+i0);
*didv=(1.0/in->Rshunt+1.0/(1e6+in->Rcontact)+didv0);//1.0/in->Rshunt-didv0;
*didphi=didphi0;
*didxil=didxil0;
*didxipl=didxipl0;
*didphir=didphir0;
*didxir=didxir0;
*didxipr=didxipr0;

/**i=i0;
*didv=didv0;//1.0/in->Rshunt-didv0;
*didphi=didphi0;
*didxil=didxil0;
*didxipl=didxipl0;
*didphir=didphir0;
*didxir=didxir0;
*didxipr=didxipr0;
*/
#endif

return;
}


double sim_voc(struct device *in)
{
struct buffer buf;
buffer_init(&buf);
char name[200];

struct sun_voc config;
sun_voc_load_config(&config,in);

struct dynamic_store store;
dump_dynamic_init(&store,in);

if (config.sun_voc_single_point)
{
	light_solve_and_update(in,&(in->mylight),in->Psun,0.0,0.0);


	solve_sun_voc(in);
	printf("final Psun= %le Voc=%le\n",in->Psun,get_equiv_V(in));


	double n_voc=get_total_np(in);
	double r_voc=get_avg_Rn(in);//get_avg_recom(in);
	double J=get_avg_J(in);
	double cur=get_I(in);
	FILE *out;
	out=fopena(in->outputpath,"./sim_info.dat","w");
	fprintf(out,"#voc\n%le\n",get_equiv_V(in));
	fprintf(out,"#voc_nt\n%le\n",get_n_trapped_charge(in));
	fprintf(out,"#voc_pt\n%le\n",get_p_trapped_charge(in));
	fprintf(out,"#voc_nf\n%le\n",get_free_n_charge(in));
	fprintf(out,"#voc_pf\n%le\n",get_free_p_charge(in));
	fprintf(out,"#voc_np_tot\n%le\n",n_voc);
	fprintf(out,"#voc_tau\n%le\n",n_voc/r_voc);
	fprintf(out,"#voc_R\n%le\n",r_voc);
	fprintf(out,"#voc_Jr\n%le\n",r_voc*in->ylen);
	fprintf(out,"#voc_J\n%le\n",J);
	fprintf(out,"#voc_J_to_Jr\n%le\n",fabs(J)/(Q*r_voc*in->ylen));
	fprintf(out,"#voc_i\n%le\n",fabs(cur));
	fprintf(out,"#end");

	fclose(out);

	dump_dynamic_add_data(&store,in,in->time);
	dump_write_to_disk(in);
}else
{

	struct istruct sun_voc_out;
	inter_init(&sun_voc_out);

	double sun=config.sun_voc_Psun_start;
	do
	{
		in->Psun=sun;
		light_solve_and_update(in,&(in->mylight),in->Psun,0.0,0.0);

		in->newton_aux=&newton_aux_voc;
		solve_sun_voc(in);
		printf("final Psun= %le Voc=%le\n",in->Psun,get_equiv_V(in));
		inter_append(&sun_voc_out,in->Psun,get_equiv_V(in));

		if (sun>config.sun_voc_Psun_stop)
		{
			break;
		}
		dump_dynamic_add_data(&store,in,in->time);
		sun*=config.sun_voc_Psun_mul;

		dump_write_to_disk(in);
		plot_now(in,"sun_voc.plot");
		stop_start(in);
	}while(1);

	buffer_malloc(&buf);
	sprintf(name,"%s","sun_voc.dat");
	buf.y_mul=1.0;
	buf.x_mul=1.0;
	strcpy(buf.title,"Sun - Voc");
	strcpy(buf.type,"xy");
	strcpy(buf.x_label,"Suns");
	strcpy(buf.y_label,"V_{oc}");
	strcpy(buf.x_units,"Volts");
	strcpy(buf.y_units,"(Suns)");
	buf.logscale_x=1;
	buf.logscale_y=1;
	buffer_add_info(&buf);
	buffer_add_xy_data(&buf,sun_voc_out.x, sun_voc_out.data, sun_voc_out.len);
	buffer_dump(in->outputpath,name,&buf);
	buffer_free(&buf);

	inter_free(&sun_voc_out);
	dump_dynamic_save(in->outputpath,&store);
	dump_dynamic_free(&store);

}

/*FILE *test=fopen("./test.dat","w");
int band;
for (band=0;band<in->srh_bands;band++)
{
	fprintf(test,"%d %le\n",band,in->nt[10][band]);

}
fclose(test);
double E=-1.0;
FILE *test2=fopen("./test2.dat","w");
do
{
	fprintf(test2,"%le ",E);
	for (band=0;band<in->srh_bands;band++)
	{
		fprintf(test2,"%le ",get_n_pop_srh(E,in->Te[0],band,0));
	}

fprintf(test2,"\n");
E+=1e-3;
}while(E<0.0);

fclose(test2);*/
return 0.0;
}

void sun_voc_load_config(struct sun_voc* in,struct device *dev)
{
struct inp_file inp;
inp_init(&inp);
inp_load_from_path(&inp,dev->inputpath,"sun_voc.inp");
inp_check(&inp,1.0);
inp_search_int(&inp,&(in->sun_voc_single_point),"#sun_voc_single_point");
inp_search_double(&inp,&(in->sun_voc_Psun_start),"#sun_voc_Psun_start");
inp_search_double(&inp,&(in->sun_voc_Psun_stop),"#sun_voc_Psun_stop");
inp_search_double(&inp,&(in->sun_voc_Psun_mul),"#sun_voc_Psun_mul");
inp_free(&inp);

}
