// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include <math.h>

static int unused __attribute__((unused));

#ifdef enable_time
static int mesh_len=0;
static double mesh[10];
static double mesh_dt[10];
static double mesh_dt_mull[10];
#endif

void time_load_mesh(struct device *in)
{
#ifdef enable_time
int i;
FILE *config;
config=fopena(in->inputpath,"time_mesh.inp","r");
if (config==NULL)
{
	ewe("time_mesh.inp not found\n");
}
unused=fscanf(config,"%d",&mesh_len);
for (i=0;i<mesh_len;i++)
{
unused=fscanf(config,"%le %le %le",&mesh[i],&mesh_dt[i],&mesh_dt_mull[i]);
}
fclose(config);
#endif
}

void time_update_dt(struct device *in)
{
#ifdef enable_time
int i;
static double old_dt=-1.0;
double dt=mesh_dt[0];

for (i=0;i<mesh_len;i++)
{
	if (in->time<mesh[i])
	{
		dt=mesh_dt[i];
		break;
	}
}

if (old_dt==dt)
{
	in->dt*=mesh_dt_mull[i];
}else
{
	in->dt=dt;
}

old_dt=dt;
#endif
}

void time_init(struct device *in)
{
#ifdef enable_time
time_load_mesh(in);
int i;
int band;

for (i=0;i<in->ymeshpoints;i++)
{

	for (band=0;band<in->srh_bands;band++)
	{
		in->ntlast[i][band]=in->nt[i][band];
		in->ptlast[i][band]=in->pt[i][band];
	}

in->nlast[i]=in->n[i];
in->plast[i]=in->p[i];

}
#endif
}

void device_timestep(struct device *in)
{
#ifdef enable_time
int i;
int band;

for (i=0;i<in->ymeshpoints;i++)
{
	in->nlast[i]=in->n[i];
	in->plast[i]=in->p[i];

	for (band=0;band<in->srh_bands;band++)
	{
		in->ntlast[i][band]=in->nt[i][band];
		in->ptlast[i][band]=in->pt[i][band];
	}
}
in->Vapplied_last=in->Vapplied;
in->VCext_last=in->VCext;
in->Ilast=get_I(in);
in->time+=in->dt;
#endif
//if (in->time>1e-6) exit(0);
}

